@@ -1,7 +1,7 @@
 # kate: default-dictionary en_US
 
 ## This file is part of the 'stringi' package for R.
-## Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+## Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
 ## All rights reserved.
 ##
 ## Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,7 @@
 
 
 #' @title
-#' Replace Occurrences of a Pattern
+#' Replace Pattern Occurrences
 #'
 #' @description
 #' These functions replace, with the given replacement string, every/first/last
@@ -46,7 +46,7 @@
 #' each match is replaced in the result by the replacement string.
 #'
 #' However, for \code{stri_replace_all*}, if \code{vectorize_all} is \code{FALSE},
-#' the each substring matching any of the supplied \code{pattern}s
+#' then each substring matching any of the supplied \code{pattern}s
 #' is replaced by a corresponding \code{replacement} string.
 #' In such a case, the vectorization is over \code{str},
 #' and - independently - over \code{pattern} and \code{replacement}.
@@ -63,6 +63,9 @@
 #' escape it with a backslash.
 #' Moreover, \code{${name}} are used for named capture groups.
 #'
+#' Note that \code{stri_replace_last_regex} searches from start to end,
+#' but skips overlapping matches, see the example below.
+#'
 #' \code{stri_replace}, \code{stri_replace_all}, \code{stri_replace_first},
 #' and \code{stri_replace_last} are convenience functions; they just call
 #' \code{stri_replace_*_*} variants, depending on the arguments used.
@@ -113,8 +116,8 @@
 #' stri_replace_all_regex('123|456|789', '(\\p{N}).(\\p{N})', '$2-$1')
 #' stri_replace_all_regex(c('stringi R', 'REXAMINE', '123'), '( R|R.)', ' r ')
 #'
+#' # named capture groups are available since ICU 55
 #' \dontrun{
-#' # named capture groups available since ICU 55
 #' stri_replace_all_regex('words 123 and numbers 456',
 #'    '(?<numbers>[0-9]+)', '!${numbers}!')
 #' }
@@ -131,6 +134,11 @@
 #' stri_replace_all_regex('The quicker brown fox jumped over the lazy dog.',
 #'      '\\b'%s+%c('quick', 'brown', 'fox')%s+%'\\b', c('slow',  'black', 'bear'), vectorize_all=FALSE)
 #'
+#' # Searching for the last occurrence:
+#' # Note the difference - regex searches left to right, with no overlaps.
+#' stri_replace_last_fixed("agAGA", "aga", "*", case_insensitive=TRUE)
+#' stri_replace_last_regex("agAGA", "aga", "*", case_insensitive=TRUE)
+#'
 #' @family search_replace
 #' @export
 #' @rdname stri_replace
@@ -347,3 +355,22 @@ stri_replace_last_regex <- function(str, pattern, replacement, ..., opts_regex =
         opts_regex <- do.call(stri_opts_regex, as.list(c(opts_regex, ...)))
     .Call(C_stri_replace_last_regex, str, pattern, replacement, opts_regex)
 }
+
+
+#' Convert gsub-Style Replacement Strings
+#'
+#' @description
+#' Converts a \code{\link[base]{gsub}}-style replacement strings
+#' to those which can be used in \code{\link{stri_replace}}.
+#' In particular, \code{$} becomes \code{\\$} and \code{\\1} becomes \code{$1}.
+#'
+#' @param x character vector
+#'
+#' @return Returns a character vector.
+#'
+#' @family search_replace
+#' @export
+stri_replace_rstr <- function(x)
+{
+    .Call(C_stri_replace_rstr, x)
+}
