@@ -1813,7 +1813,7 @@ int32_t FixedDecimal::decimals(double n) {
 
     // Slow path, convert with sprintf, parse converted output.
     char  buf[30] = {0};
-    sprintf(buf, "%1.15e", n);
+    snprintf(buf, 30, "%1.15e", n);    // MAREK's patch sprintf -> snprintf
     // formatted number looks like this: 1.234567890123457e-01
     int exponent = atoi(buf+18);
     int numFractionDigits = 15;
