@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2022, Marek Gagolewski <https://www.gagolewski.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,10 @@
 #ifndef __stri_container_base_h
 #define __stri_container_base_h
 
+#include "stri_external.h"
+#include "stri_exception.h"
+
+
 
 /**
  * Base class for all StriContainers
