@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,7 +63,7 @@ public:
 #endif
         R_len_t ndata = LENGTH(rvec);
         this->init_Base(ndata, _nrecycle, true);
-        this->data = INTEGER(rvec);
+        this->data = INTEGER(rvec);  // TODO: ALTREP will be problematic?
     }
 
     //  StriContainerInteger(StriContainerInteger& container); // default-shallow
@@ -85,6 +85,7 @@ public:
 
 
     /** get the vectorized ith element
+     *
      * @param i index
      * @return integer
      */
@@ -97,6 +98,20 @@ public:
 #endif
         return (data[i%n]);
     }
+
+
+    /** get the vectorized ith element, no NA check here
+     *
+     * @param i index
+     * @return integer
+     */
+    inline int getNAble(R_len_t i) const {
+#ifndef NDEBUG
+        if (i < 0 || i >= nrecycle)
+            throw StriException("StriContainerInteger::get(): INDEX OUT OF BOUNDS");
+#endif
+        return (data[i%n]);
+    }
 };
 
 #endif
