@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@ StriContainerListInt::StriContainerListInt(SEXP rstr)
 {
     this->data = NULL;
 
-    if (isNull(rstr)) {
+    if (Rf_isNull(rstr)) {
         this->init_Base(1, 1, true);
         this->data = new IntVec[this->n]; // 1 vector, NA/NULL
         if (!this->data) throw StriException(MSG__MEM_ALLOC_ERROR);
@@ -64,7 +64,7 @@ StriContainerListInt::StriContainerListInt(SEXP rstr)
         this->init_Base(1, 1, true);
         this->data = new IntVec[this->n];
         if (!this->data) throw StriException(MSG__MEM_ALLOC_ERROR);
-        this->data[0].initialize((const int*)INTEGER(rstr), LENGTH(rstr)); // shallow copy
+        this->data[0].initialize((const int*)INTEGER(rstr), LENGTH(rstr)); // shallow copy // TODO: ALTREP will be problematic?
     }
     else // if (Rf_isVectorList(rstr)) -- args already checked
     {
@@ -74,8 +74,8 @@ StriContainerListInt::StriContainerListInt(SEXP rstr)
         if (!this->data) throw StriException(MSG__MEM_ALLOC_ERROR);
         for (R_len_t i=0; i<this->n; ++i) {
             SEXP cur = VECTOR_ELT(rstr, i);
-            if (!isNull(cur))
-                this->data[i].initialize((const int*)INTEGER(cur), LENGTH(cur)); // shallow copy
+            if (!Rf_isNull(cur))
+                this->data[i].initialize((const int*)INTEGER(cur), LENGTH(cur)); // shallow copy // TODO: ALTREP will be problematic?
             // else leave as-is, i.e., NULL/NA
         }
     }
