@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,7 +53,7 @@
  */
 SEXP stri_rand_shuffle(SEXP str)
 {
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
     R_len_t n = LENGTH(str);
 
     GetRNGstate();
@@ -91,9 +91,10 @@ SEXP stri_rand_shuffle(SEXP str)
         }
 
         if (c < 0) {
-            Rf_warning(MSG__INVALID_UTF8);
-            SET_STRING_ELT(ret, i, NA_STRING);
-            continue;
+            throw StriException(MSG__INVALID_UTF8);
+//             Rf_warning(...);
+//             SET_STRING_ELT(ret, i, NA_STRING);
+//             continue;
         }
 
         // do shuffle buf1 at pos 0..k-1: (Fisher-Yates shuffle)
@@ -150,8 +151,8 @@ SEXP stri_rand_shuffle(SEXP str)
 SEXP stri_rand_strings(SEXP n, SEXP length, SEXP pattern)
 {
     int n_val = stri__prepare_arg_integer_1_notNA(n, "n");
-    PROTECT(length    = stri_prepare_arg_integer(length, "length"));
-    PROTECT(pattern   = stri_prepare_arg_string(pattern, "pattern"));
+    PROTECT(length    = stri__prepare_arg_integer(length, "length"));
+    PROTECT(pattern   = stri__prepare_arg_string(pattern, "pattern"));
 
     if (n_val < 0) n_val = 0; /* that's not NA for sure now */
 
