@@ -1,5 +1,5 @@
 /* This file is part of the 'stringi' project.
- * Copyright (c) 2013-2020, Marek Gagolewski <https://www.gagolewski.com>
+ * Copyright (c) 2013-2021, Marek Gagolewski <https://www.gagolewski.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,17 +35,24 @@
 #include "stri_string8buf.h"
 #include <stdexcept>
 
-/***
+
+/**
  * used both in stri_sub and stri_sub_replacement
  *
  * @return number of objects PROTECTEd
+ *
+ * @version ??? (Marek Gagolewski, 20??-??-??)
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-30) allow (from,length) matrices
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08) use_matrix
  */
 R_len_t stri__sub_prepare_from_to_length(SEXP& from, SEXP& to, SEXP& length,
         R_len_t& from_len, R_len_t& to_len, R_len_t& length_len,
-        int*& from_tab, int*& to_tab, int*& length_tab)
+        int*& from_tab, int*& to_tab, int*& length_tab, bool use_matrix_1)
 {
     R_len_t sub_protected = 0;
-    bool from_ismatrix = Rf_isMatrix(from);
+    bool from_ismatrix = use_matrix_1 && Rf_isMatrix(from);
     if (from_ismatrix) {
         SEXP t;
         PROTECT(t = Rf_getAttrib(from, R_DimSymbol));
@@ -56,24 +63,48 @@ R_len_t stri__sub_prepare_from_to_length(SEXP& from, SEXP& to, SEXP& length,
             UNPROTECT(1); // t
             Rf_error(MSG__ARG_EXPECTED_MATRIX_WITH_GIVEN_COLUMNS, "from", 2);
         }
-        UNPROTECT(1); // t
+        UNPROTECT(1);  // t
     }
 
     sub_protected++;
-    PROTECT(from = stri_prepare_arg_integer(from, "from"));
+    PROTECT(from = stri__prepare_arg_integer(from, "from"));
     /* may remove R_DimSymbol */
 
     if (from_ismatrix) {
-        from_len      = LENGTH(from)/2;
-        to_len        = from_len;
-        from_tab      = INTEGER(from);
-        to_tab        = from_tab+from_len;
+        bool fromlength_matrix = false;
+        SEXP t;
+        PROTECT(t = Rf_getAttrib(from, R_DimNamesSymbol));
+        if (!Rf_isNull(t)) {
+            SEXP t2;
+            PROTECT(t2 = VECTOR_ELT(t, 1));
+            if (
+                isString(t2) && LENGTH(t2) == 2 &&
+                strcmp("length", CHAR(STRING_ELT(t2, 1))) == 0
+            ) {
+                fromlength_matrix = true;
+            }
+            UNPROTECT(1);  // t2
+        }
+        UNPROTECT(1);  // t
+
+        if (fromlength_matrix) {
+            from_len      = LENGTH(from)/2;
+            length_len    = from_len;
+            from_tab      = INTEGER(from);
+            length_tab    = from_tab+from_len;
+        }
+        else {
+            from_len      = LENGTH(from)/2;
+            to_len        = from_len;
+            from_tab      = INTEGER(from);
+            to_tab        = from_tab+from_len;
+        }
         //PROTECT(to); /* fake - not to provoke stack imbalance */
         //PROTECT(length); /* fake - not to provoke stack imbalance */
     }
-    else if (isNull(length)) {
+    else if (Rf_isNull(length)) {
         sub_protected++;
-        PROTECT(to    = stri_prepare_arg_integer(to, "to"));
+        PROTECT(to    = stri__prepare_arg_integer(to, "to"));
         from_len      = LENGTH(from);
         from_tab      = INTEGER(from);
         to_len        = LENGTH(to);
@@ -82,7 +113,7 @@ R_len_t stri__sub_prepare_from_to_length(SEXP& from, SEXP& to, SEXP& length,
     }
     else {
         sub_protected++;
-        PROTECT(length= stri_prepare_arg_integer(length, "length"));
+        PROTECT(length= stri__prepare_arg_integer(length, "length"));
         from_len      = LENGTH(from);
         from_tab      = INTEGER(from);
         length_len    = LENGTH(length);
@@ -99,7 +130,7 @@ R_len_t stri__sub_prepare_from_to_length(SEXP& from, SEXP& to, SEXP& length,
 }
 
 
-/***
+/**
  * used both in stri_sub and stri_sub_replacement
  */
 inline void stri__sub_get_indices(StriContainerUTF8_indexable& str_cont, R_len_t& i,
@@ -137,32 +168,40 @@ inline void stri__sub_get_indices(StriContainerUTF8_indexable& str_cont, R_len_t
  * @return character vector
  *
  * @version 0.1-?? (Bartek Tartanus)
- *          stri_sub
+ *    stri_sub
  *
  * @version 0.1-?? (Marek Gagolewski)
- *          use StriContainerUTF8 and stri__UChar32_to_UTF8_index
+ *    Use StriContainerUTF8 and stri__UChar32_to_UTF8_index
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-01)
- *          use StriContainerUTF8's UChar32-to-UTF8 index
+ *    Use StriContainerUTF8's UChar32-to-UTF8 index
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-16)
- *          make StriException-friendly
+ *    Make StriException-friendly
  *
  * @version 0.2-1 (Marek Gagolewski, 2014-03-20)
- *          Use StriContainerUTF8_indexable
+ *    Use StriContainerUTF8_indexable
  *
  * @version 0.2-1 (Marek Gagolewski, 2014-04-03)
- *          Use stri__sub_prepare_from_to_length()
+ *    Use stri__sub_prepare_from_to_length()
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-04)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  *
  * @version 0.5-9003 (Marek Gagolewski, 2015-08-05)
  *    Bugfix #183: floating point exception when to or length is an empty vector
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-28)
+ *    Negative length yields NA
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08)
+ *    use_matrix, ignore_negative_length
  */
-SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
+SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length, SEXP use_matrix, SEXP ignore_negative_length)
 {
-    PROTECT(str = stri_prepare_arg_string(str, "str"));
+    PROTECT(str = stri__prepare_arg_string(str, "str"));
+    bool use_matrix_1 = stri__prepare_arg_logical_1_notNA(use_matrix, "use_matrix");
+    bool ignore_negative_length_1 = stri__prepare_arg_logical_1_notNA(ignore_negative_length, "ignore_negative_length");
 
     R_len_t str_len       = LENGTH(str);
     R_len_t from_len      = 0;
@@ -174,7 +213,7 @@ SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
 
     R_len_t sub_protected =  1+  /* how many objects to PROTECT on ret? */
                              stri__sub_prepare_from_to_length(from, to, length,
-                                     from_len, to_len, length_len, from_tab, to_tab, length_tab);
+                                     from_len, to_len, length_len, from_tab, to_tab, length_tab, use_matrix_1);
 
     R_len_t vectorize_len = stri__recycling_rule(true, 3,
                             str_len, from_len, (to_len>length_len)?to_len:length_len);
@@ -189,6 +228,7 @@ SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
     SEXP ret;
     STRI__PROTECT(ret = Rf_allocVector(STRSXP, vectorize_len));
 
+    R_len_t num_negative_length = 0;
     for (R_len_t i = str_cont.vectorize_init();
             i != str_cont.vectorize_end();
             i = str_cont.vectorize_next(i))
@@ -201,10 +241,16 @@ SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
         }
 
         if (length_tab) {
-            if (cur_to <= 0) {
+            if (cur_to == 0) {
                 SET_STRING_ELT(ret, i, R_BlankString);
                 continue;
             }
+            else if (cur_to < 0) {
+                SET_STRING_ELT(ret, i, NA_STRING);
+                num_negative_length++;
+                continue;
+            }
+
             cur_to = cur_from + cur_to - 1;
             if (cur_from < 0 && cur_to >= 0) cur_to = -1;
         }
@@ -225,6 +271,29 @@ SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
         }
     }
 
+    if (num_negative_length > 0 && ignore_negative_length_1) {
+        // stringx: ignore items corresponding to length<0
+        STRI_ASSERT(length_tab)
+
+        SEXP ret_old = ret;
+        STRI__PROTECT(ret = Rf_allocVector(STRSXP, vectorize_len-num_negative_length));
+        R_len_t k = 0;
+        for (R_len_t i = str_cont.vectorize_init();
+            i != str_cont.vectorize_end();
+            i = str_cont.vectorize_next(i))
+        {
+            R_len_t cur_from     = from_tab[i % from_len];
+            R_len_t cur_to       = length_tab[i % length_len];
+            if (!str_cont.isNA(i) && cur_from != NA_INTEGER && cur_to != NA_INTEGER && cur_to < 0) {
+                // ignore
+            }
+            else {
+                SET_STRING_ELT(ret, k, STRING_ELT(ret_old, i));
+                ++k;
+            }
+        }
+    }
+
     STRI__UNPROTECT_ALL
     return ret;
     STRI__ERROR_HANDLER_END(;/* nothing special to be done on error */)
@@ -273,12 +342,19 @@ SEXP stri_sub(SEXP str, SEXP from, SEXP to, SEXP length)
  *
  * @version 1.4.3 (Marek Gagolewski, 2019-03-12)
  *    #346: na_omit for `value`
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-28)
+ *    negative length does not alter input
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08)
+ *    use_matrix
  */
-SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_na, SEXP value)
+SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_na, SEXP value, SEXP use_matrix)
 {
-    PROTECT(str   = stri_prepare_arg_string(str, "str"));
-    PROTECT(value = stri_prepare_arg_string(value, "value"));
+    PROTECT(str   = stri__prepare_arg_string(str, "str"));
+    PROTECT(value = stri__prepare_arg_string(value, "value"));
     bool omit_na_1 = stri__prepare_arg_logical_1_notNA(omit_na, "omit_na");
+    bool use_matrix_1 = stri__prepare_arg_logical_1_notNA(use_matrix, "use_matrix");
 
     R_len_t value_len     = LENGTH(value);
     R_len_t str_len       = LENGTH(str);
@@ -291,7 +367,7 @@ SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_n
 
     R_len_t sub_protected =  2+ /* how many objects to PROTECT on ret? */
                              stri__sub_prepare_from_to_length(from, to, length,
-                                     from_len, to_len, length_len, from_tab, to_tab, length_tab);
+                                     from_len, to_len, length_len, from_tab, to_tab, length_tab, use_matrix_1);
 
     R_len_t vectorize_len = stri__recycling_rule(true, 4,
                             str_len, value_len, from_len, (to_len>length_len)?to_len:length_len);
@@ -319,6 +395,7 @@ SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_n
             SET_STRING_ELT(ret, i, NA_STRING);
             continue;
         }
+
         if (cur_from == NA_INTEGER || cur_to == NA_INTEGER || value_cont.isNA(i)) {
             if (omit_na_1) {
                 SET_STRING_ELT(ret, i, str_cont.toR(i));
@@ -329,6 +406,11 @@ SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_n
             continue;
         }
 
+        if (!to_tab && cur_to/*length*/ < 0) {  // so not NA
+            SET_STRING_ELT(ret, i, str_cont.toR(i));
+            continue;
+        }
+
         if (length_tab) {
             if (cur_to <= 0) {
                 // SET_STRING_ELT(ret, i, R_BlankString);
@@ -382,13 +464,20 @@ SEXP stri_sub_replacement(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_n
  *
  * @version 1.3.2 (Marek Gagolewski, 2019-02-21)
  *    #30: new function
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-28)
+ *    negative length yields NA
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08)
+ *    use_matrix, ignore_negative_length
  */
-SEXP stri_sub_all(SEXP str, SEXP from, SEXP to, SEXP length)
+SEXP stri_sub_all(SEXP str, SEXP from, SEXP to, SEXP length, SEXP use_matrix, SEXP ignore_negative_length)
 {
-    PROTECT(str    = stri_prepare_arg_string(str, "str"));
-    PROTECT(from   = stri_prepare_arg_list(from, "from"));
-    PROTECT(to     = stri_prepare_arg_list(to, "to"));
-    PROTECT(length = stri_prepare_arg_list(length, "length"));
+    PROTECT(str    = stri__prepare_arg_string(str, "str"));
+    PROTECT(from   = stri__prepare_arg_list(from, "from"));
+    PROTECT(to     = stri__prepare_arg_list(to, "to"));
+    PROTECT(length = stri__prepare_arg_list(length, "length"));
+//     bool use_matrix_1 = stri__prepare_arg_logical_1_notNA(use_matrix, "use_matrix");
 
     R_len_t str_len       = LENGTH(str);
     R_len_t from_len      = LENGTH(from);
@@ -397,10 +486,10 @@ SEXP stri_sub_all(SEXP str, SEXP from, SEXP to, SEXP length)
 
 
     R_len_t vectorize_len;
-    if (!isNull(to))
+    if (!Rf_isNull(to))
         vectorize_len = stri__recycling_rule(true, 3,
                                              str_len, from_len, LENGTH(to));
-    else if (!isNull(length))
+    else if (!Rf_isNull(length))
         vectorize_len = stri__recycling_rule(true, 3,
                                              str_len, from_len, LENGTH(length));
     else
@@ -424,17 +513,20 @@ SEXP stri_sub_all(SEXP str, SEXP from, SEXP to, SEXP length)
         SET_STRING_ELT(str_tmp, 0, tmp);
         UNPROTECT(1); //tmp
 
-        if (!isNull(to)) {
-            PROTECT(tmp = stri_sub(str_tmp,
-                                   VECTOR_ELT(from, i%from_len), VECTOR_ELT(to, i%LENGTH(to)), R_NilValue));
+        if (!Rf_isNull(to)) {
+            PROTECT(tmp = stri_sub(
+                str_tmp, VECTOR_ELT(from, i%from_len), VECTOR_ELT(to, i%LENGTH(to)), R_NilValue, use_matrix, ignore_negative_length
+            ));
         }
-        else if (!isNull(length)) {
-            PROTECT(tmp = stri_sub(str_tmp,
-                                   VECTOR_ELT(from, i%from_len), R_NilValue, VECTOR_ELT(length, i%LENGTH(length))));
+        else if (!Rf_isNull(length)) {
+            PROTECT(tmp = stri_sub(
+                str_tmp, VECTOR_ELT(from, i%from_len), R_NilValue, VECTOR_ELT(length, i%LENGTH(length)), use_matrix, ignore_negative_length
+            ));
         }
         else {
-            PROTECT(tmp = stri_sub(str_tmp,
-                                   VECTOR_ELT(from, i%from_len), R_NilValue, R_NilValue));
+            PROTECT(tmp = stri_sub(
+                str_tmp, VECTOR_ELT(from, i%from_len), R_NilValue, R_NilValue, use_matrix, ignore_negative_length
+            ));
         }
 
         SET_VECTOR_ELT(ret, i, tmp);
@@ -457,14 +549,20 @@ SEXP stri_sub_all(SEXP str, SEXP from, SEXP to, SEXP length)
  * @version 1.4.4 (Marek Gagolewski, 2019-03-13)-
  *    #348: UBSAN runtime error: null pointer passed as argument 1,
  *     which is declared to never be null
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-28)
+ *    negative length does not alter input
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08)
+ *    use_matrix
  */
-SEXP stri__sub_replacement_all_single(SEXP curs,
-                                      SEXP from, SEXP to, SEXP length, bool omit_na_1, SEXP value)
-{
+SEXP stri__sub_replacement_all_single(
+    SEXP curs,
+    SEXP from, SEXP to, SEXP length, bool omit_na_1, bool use_matrix_1, SEXP value
+) {
     // curs is a CHARSXP in UTF-8
 
-    PROTECT(value = stri_enc_toutf8(value,
-                                    Rf_ScalarLogical(FALSE), Rf_ScalarLogical(FALSE)));
+    PROTECT(value = stri_enc_toutf8(value, Rf_ScalarLogical(FALSE), Rf_ScalarLogical(FALSE)));
     R_len_t value_len     = LENGTH(value);
 
     R_len_t from_len      = 0; // see below
@@ -476,7 +574,7 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
 
     R_len_t sub_protected = 1+ /* how many objects to PROTECT on ret? */
                             stri__sub_prepare_from_to_length(from, to, length,
-                                    from_len, to_len, length_len, from_tab, to_tab, length_tab);
+                                    from_len, to_len, length_len, from_tab, to_tab, length_tab, use_matrix_1);
 
     R_len_t vectorize_len = stri__recycling_rule(true, 2, // does not care about value_len
                             from_len, (to_len>length_len)?to_len:length_len);
@@ -491,11 +589,7 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
         return NA_STRING;
     }
 
-    if (vectorize_len % value_len != 0)
-        Rf_warning(MSG__WARN_RECYCLING_RULE2);
-
-
-    const char* curs_s = CHAR(curs); // already in UTF-8
+    const char* curs_s = CHAR(curs); // already in UTF-8  // TODO: ALTREP will be problematic?
     R_len_t curs_n = LENGTH(curs);
 
     // first check for NAs....
@@ -518,6 +612,8 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
         }
     }
 
+
+
     // get the number of code points in curs, if required (for negative indexes)
     R_len_t curs_m = -1;
     if (IS_ASCII(curs)) curs_m = curs_n;
@@ -533,17 +629,25 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
     STRI__ERROR_HANDLER_BEGIN(sub_protected)
     std::vector<char> buf; // convenience >> speed
 
+    R_len_t num_replaced = 0;
     R_len_t last_pos = 0;
     R_len_t byte_pos = 0;
     for (R_len_t i=0; i<vectorize_len; ++i) {
         R_len_t cur_from     = from_tab[i % from_len];
         R_len_t cur_to       = (to_tab)?to_tab[i % to_len]:length_tab[i % length_len];
 
-        if (cur_from == NA_INTEGER || cur_to == NA_INTEGER || STRING_ELT(value, i%value_len) == NA_STRING) {
-            // omit_na is true
+        if (
+            cur_from == NA_INTEGER ||
+            cur_to == NA_INTEGER ||
+            STRING_ELT(value, i%value_len) == NA_STRING ||
+            (!to_tab && cur_to/*length*/ < 0)
+        ) {
+            // omit_na is true or negative length
             continue;
         }
 
+        num_replaced++;
+
         if (cur_from < 0) cur_from = curs_m+cur_from+1;
         if (cur_from <= 0) cur_from = 1;
         cur_from--; // 1-based -> 0-based index
@@ -585,7 +689,7 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
 
         // then, copy the corresponding replacement string
         SEXP value_cur = STRING_ELT(value, i%value_len);
-        const char* value_s = CHAR(value_cur);
+        const char* value_s = CHAR(value_cur);  // TODO: ALTREP will be problematic?
         R_len_t value_n = LENGTH(value_cur);
         if (value_n > 0) {
             R_len_t buf_size = buf.size();
@@ -613,6 +717,10 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
         memcpy(buf.data()+buf_size, curs_s+byte_pos, curs_n-byte_pos);
     }
 
+    // only warn if not NA
+    if (num_replaced > 0 && vectorize_len % value_len != 0)
+        Rf_warning(MSG__WARN_RECYCLING_RULE2);
+
     SEXP ret;
     STRI__PROTECT(ret = Rf_mkCharLenCE(buf.data(), buf.size(), CE_UTF8));
     STRI__UNPROTECT_ALL
@@ -639,16 +747,23 @@ SEXP stri__sub_replacement_all_single(SEXP curs,
  *
  * @version 1.4.3 (Marek Gagolewski, 2019-03-12)
  *    #346: na_omit for `value`
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-28)
+ *    negative length does not alter input
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-07-08)
+ *    use_matrix
  */
-SEXP stri_sub_replacement_all(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_na, SEXP value)
+SEXP stri_sub_replacement_all(SEXP str, SEXP from, SEXP to, SEXP length, SEXP omit_na, SEXP value, SEXP use_matrix)
 {
-    //PROTECT(str    = stri_prepare_arg_string(str, "str"));
+    //PROTECT(str    = stri__prepare_arg_string(str, "str"));
     PROTECT(str = stri_enc_toutf8(str, Rf_ScalarLogical(FALSE), Rf_ScalarLogical(FALSE)));
-    PROTECT(from   = stri_prepare_arg_list(from, "from"));
-    PROTECT(to     = stri_prepare_arg_list(to, "to"));
-    PROTECT(length = stri_prepare_arg_list(length, "length"));
-    PROTECT(value  = stri_prepare_arg_list(value, "value"));
+    PROTECT(from   = stri__prepare_arg_list(from, "from"));
+    PROTECT(to     = stri__prepare_arg_list(to, "to"));
+    PROTECT(length = stri__prepare_arg_list(length, "length"));
+    PROTECT(value  = stri__prepare_arg_list(value, "value"));
     bool omit_na_1 = stri__prepare_arg_logical_1_notNA(omit_na, "omit_na");
+    bool use_matrix_1 = stri__prepare_arg_logical_1_notNA(use_matrix, "use_matrix");
 
     R_len_t str_len       = LENGTH(str);
     R_len_t from_len      = LENGTH(from);
@@ -656,10 +771,10 @@ SEXP stri_sub_replacement_all(SEXP str, SEXP from, SEXP to, SEXP length, SEXP om
 
 
     R_len_t vectorize_len;
-    if (!isNull(to))
+    if (!Rf_isNull(to))
         vectorize_len = stri__recycling_rule(true, 4,
                                              str_len, from_len, value_len, LENGTH(to));
-    else if (!isNull(length))
+    else if (!Rf_isNull(length))
         vectorize_len = stri__recycling_rule(true, 4,
                                              str_len, from_len, value_len, LENGTH(length));
     else
@@ -682,23 +797,23 @@ SEXP stri_sub_replacement_all(SEXP str, SEXP from, SEXP to, SEXP length, SEXP om
             continue;
         }
 
-        if (!isNull(to)) {
+        if (!Rf_isNull(to)) {
             PROTECT(tmp = stri__sub_replacement_all_single(curs,
                           VECTOR_ELT(from, i%from_len),
                           VECTOR_ELT(to, i%LENGTH(to)), R_NilValue,
-                          omit_na_1, VECTOR_ELT(value, i%value_len)));
+                          omit_na_1, use_matrix_1, VECTOR_ELT(value, i%value_len)));
         }
-        else if (!isNull(length)) {
+        else if (!Rf_isNull(length)) {
             PROTECT(tmp = stri__sub_replacement_all_single(curs,
                           VECTOR_ELT(from, i%from_len),
                           R_NilValue, VECTOR_ELT(length, i%LENGTH(length)),
-                          omit_na_1, VECTOR_ELT(value, i%value_len)));
+                          omit_na_1, use_matrix_1, VECTOR_ELT(value, i%value_len)));
         }
         else {
             PROTECT(tmp = stri__sub_replacement_all_single(curs,
                           VECTOR_ELT(from, i%from_len),
                           R_NilValue, R_NilValue,
-                          omit_na_1, VECTOR_ELT(value, i%value_len)));
+                          omit_na_1, use_matrix_1, VECTOR_ELT(value, i%value_len)));
         }
 
         SET_STRING_ELT(ret, i, tmp);
